package com.jiemoapp.gpuimage;

import android.hardware.Camera;
import android.opengl.GLES20;
import android.opengl.Matrix;
import android.util.Log;

import java.nio.BufferOverflowException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL10;

public class GPUImageYUVFilter extends GPUImageFilter {

    private static final float TEXTURE_NO_ROTATION[] = { 0.0f, 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f,
            0.0f, };

    //    private static final float TEXTURE_ROTATED_90[] = { 1.0f, 1.0f, 1.0f, 0.0f, 0.0f, 1.0f, 0.0f,
    //            0.0f, };
    //
    //    private static final float TEXTURE_ROTATED_180[] = { 1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f,
    //            1.0f, };
    //
    //    private static final float TEXTURE_ROTATED_270[] = { 0.0f, 0.0f, 0.0f, 1.0f, 1.0f, 0.0f, 1.0f,
    //            1.0f, };

    private static final String TAG = GPUImageYUVFilter.class.getName();

    private String filter_vertex_shader = null;

    private String filter_fragment_shader = null;

    private float textureCoordinate[] = TEXTURE_NO_ROTATION;

    private float vertices[] = new float[] { -1.0f, 1.0f, 0.0f,//
            -1.0f, -1.0f, 0.0f,//
            1.0f, 1.0f, 0.0f,//
            1.0f, -1.0f, 0.0f };

    private FloatBuffer mVerticesBuffer;

    private FloatBuffer mTextureCoordinateBuffer;

    private int mGLAttribTextureCoordinate;

    private int mGLUniformTexture;

    private int mGLUniformUvTexture;

    private int mTextureId = OpenGlUtils.NO_TEXTURE;

    private int mUvTextureId = OpenGlUtils.NO_TEXTURE;

    private int mWidth = 720;

    private int mHeight = 1280;

    private boolean hasPreview = false;

    private boolean mIsInitialized;

    private ByteBuffer mByteBuffer;

    private Object lock = new Object();

    public static final String GRAYSCALE_FRAGMENT_SHADER = "varying highp vec2 textureCoordinate;\n"
            + " \n"
            + "uniform sampler2D inputImageTexture;\n"
            + " \n"
            + "uniform sampler2D uvTexture;\n"
            + "precision mediump float;\n"
            + "void main()\n"
            + "{\n"
            + "\tvec4 y = vec4((texture2D(inputImageTexture, textureCoordinate).r - 16./255.) * 1.164);  \n"
            + "  \tvec4 u = vec4(texture2D(uvTexture, textureCoordinate).a - 128./255.);  \n"
            + "  \tvec4 v = vec4(texture2D(uvTexture, textureCoordinate).r - 128./255.);  \n"
            + "  \ty += v * vec4(1.596, -0.813, 0, 0);\n"
            + "  \ty += u * vec4(0, -0.392, 2.017, 0);  \n"
            + "  \ty.a = 1.0;\n"
            + "    gl_FragColor = y;//texture2D(inputImageTexture, textureCoordinate);\n" + "}";

    public static final String NO_FILTER_VERTEX_SHADER = "uniform mat4 uMVPMatrix;\n"
            + "attribute vec4 position;\n" + "attribute vec4 inputTextureCoordinate;\n" + " \n"
            + "varying vec2 textureCoordinate;\n" + " \n" + "void main()\n" + "{\n"
            + "    gl_Position = uMVPMatrix * position; \n"
            + "    textureCoordinate = inputTextureCoordinate.xy;\n" + "}";

    private int muMVPMatrixHandle;

    private float[] mProjMatrix = new float[16];

    private float[] mVMatrix = new float[16];

    private float[] mResultMatrix = new float[16];

    //    private float[] mVMatrixY = new float[16];

    //    private float mRato=

    public GPUImageYUVFilter(boolean userCameraYUV, String fragmentShader) {
        this(userCameraYUV, NO_FILTER_VERTEX_SHADER, fragmentShader);

    }

    public GPUImageYUVFilter(boolean userCameraYUV, String vertexShader, String fragmentShader) {
        super(userCameraYUV, vertexShader, fragmentShader);
        filter_vertex_shader = vertexShader;
        filter_fragment_shader = fragmentShader;
    }

    public GPUImageYUVFilter(String vertexShader, String fragmentShader) {
        this(false, vertexShader, fragmentShader);
    }

    public GPUImageYUVFilter() {
        this(true, GRAYSCALE_FRAGMENT_SHADER);
    }

    @Override
    public void onDraw(int textureId, FloatBuffer cubeBuffer, FloatBuffer textureBuffer) {
        if (isUseCameraYUVFilter()) {
            if (!hasPreview) {
                return;
            }
            if (!mIsInitialized) {
                onInit();
            }

            //            cubeBuffer.position(0);
            //            GLES20.glVertexAttribPointer(mGLAttribPosition, 2, GLES20.GL_FLOAT, false, 0,
            //                    cubeBuffer);
            //            GLES20.glEnableVertexAttribArray(mGLAttribPosition);
            //            textureBuffer.position(0);
            //            GLES20.glVertexAttribPointer(mGLAttribTextureCoordinate, 2, GLES20.GL_FLOAT, false, 0,
            //                    textureBuffer);
            //            GLES20.glEnableVertexAttribArray(mGLAttribTextureCoordinate);
            //            if (textureId != OpenGlUtils.NO_TEXTURE) {
            //                GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
            //                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textureId);
            //                GLES20.glUniform1i(mGLUniformTexture, 0);
            //            }

            onDrawArraysPre();

        } else {
            super.onDraw(textureId, cubeBuffer, textureBuffer);
        }
    }

    @Override
    protected void onDrawArraysPre() {
        if (isUseCameraYUVFilter()) {
            int count = mHeight * mWidth;
            if (!hasPreview || mByteBuffer == null || mByteBuffer.limit() < count) {
                //                Log.e(TAG, "==============hasPreview=" + hasPreview);
                return;
            }
            synchronized (lock) {

                GLES20.glUseProgram(mGLProgId);

                GLES20.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);

                GLES20.glVertexAttribPointer(mGLAttribPosition, 3, GLES20.GL_FLOAT, false, 0,
                        mVerticesBuffer);

                GLES20.glUniformMatrix4fv(muMVPMatrixHandle, 1, false, mResultMatrix, 0);

                GLES20.glEnableVertexAttribArray(mGLAttribPosition);

                GLES20.glVertexAttribPointer(mGLAttribTextureCoordinate, 2, GLES20.GL_FLOAT, false,
                        0, mTextureCoordinateBuffer);
                GLES20.glEnableVertexAttribArray(mGLAttribTextureCoordinate);

                mByteBuffer.position(0);
                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mTextureId);
                GLES20.glTexSubImage2D(GLES20.GL_TEXTURE_2D, 0, 0, 0, mWidth, mHeight,
                        GLES20.GL_LUMINANCE, GLES20.GL_UNSIGNED_BYTE, mByteBuffer);

                mByteBuffer.position(count);
                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mUvTextureId);
                GLES20.glTexSubImage2D(GLES20.GL_TEXTURE_2D, 0, 0, 0, mWidth >> 1, (mHeight) >> 1,
                        GLES20.GL_LUMINANCE_ALPHA, GLES20.GL_UNSIGNED_BYTE, mByteBuffer);

                GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mTextureId);
                GLES20.glUniform1i(mGLUniformTexture, 0);

                GLES20.glActiveTexture(GLES20.GL_TEXTURE1);
                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mUvTextureId);
                GLES20.glUniform1i(mGLUniformUvTexture, 1);

                GLES20.glDrawArrays(GLES20.GL_TRIANGLE_STRIP, 0, 4);
                GLES20.glDisableVertexAttribArray(mGLAttribPosition);
                GLES20.glDisableVertexAttribArray(mGLAttribTextureCoordinate);
                GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, 0);
            }
        } else {
            super.onDrawArraysPre();
        }
    }

    public void setPreviewSize(Camera.Size cameraPreviewSize) {
        this.mWidth = cameraPreviewSize.width;
        this.mHeight = cameraPreviewSize.height;
        synchronized (lock) {
            mByteBuffer = null;
        }
    }

    @Override
    public void onInit() {

        if (isUseCameraYUVFilter()) {
            if (!hasPreview) {
                return;
            }

            mVerticesBuffer = ByteBuffer.allocateDirect(vertices.length * 4)
                    .order(ByteOrder.nativeOrder()).asFloatBuffer();
            mVerticesBuffer.put(vertices).position(0);

            mTextureCoordinateBuffer = ByteBuffer.allocateDirect(textureCoordinate.length * 4)
                    .order(ByteOrder.nativeOrder()).asFloatBuffer();
            mTextureCoordinateBuffer.put(textureCoordinate).position(0);

            GLES20.glClearColor(0, 0, 0, 1);

            int textures[] = new int[2];
            GLES20.glGenTextures(2, textures, 0);
            GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[0]);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER,
                    GLES20.GL_LINEAR);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER,
                    GLES20.GL_LINEAR);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S,
                    GLES20.GL_CLAMP_TO_EDGE);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T,
                    GLES20.GL_CLAMP_TO_EDGE);

            mByteBuffer.position(0);
            GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_LUMINANCE, mWidth, mHeight, 0,
                    GLES20.GL_LUMINANCE, GLES20.GL_UNSIGNED_BYTE, mByteBuffer);

            GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, textures[1]);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER,
                    GLES20.GL_LINEAR);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER,
                    GLES20.GL_LINEAR);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S,
                    GLES20.GL_CLAMP_TO_EDGE);
            GLES20.glTexParameterf(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T,
                    GLES20.GL_CLAMP_TO_EDGE);

            mByteBuffer.position(mHeight * mWidth);
            GLES20.glTexImage2D(GLES20.GL_TEXTURE_2D, 0, GLES20.GL_LUMINANCE_ALPHA, mWidth / 2,
                    mHeight / 2, 0, GLES20.GL_LUMINANCE_ALPHA, GLES20.GL_UNSIGNED_BYTE, mByteBuffer);

            mTextureId = textures[0];
            mUvTextureId = textures[1];

            int vertexShaderHandle = GLES20.glCreateShader(GLES20.GL_VERTEX_SHADER);
            if (vertexShaderHandle != 0) {
                GLES20.glShaderSource(vertexShaderHandle, filter_vertex_shader);
                GLES20.glCompileShader(vertexShaderHandle);
                final int[] compileStatus = new int[1];
                GLES20.glGetShaderiv(vertexShaderHandle, GLES20.GL_COMPILE_STATUS, compileStatus, 0);
                if (compileStatus[0] == 0) {
                    GLES20.glDeleteShader(vertexShaderHandle);
                    Log.d("OpenGLES",
                            "Compilation\n" + GLES20.glGetShaderInfoLog(vertexShaderHandle));
                    vertexShaderHandle = 0;
                }
            }

            if (vertexShaderHandle == 0) {
                Log.e(TAG, "filter_vertex_shader=\n" + filter_vertex_shader);
                throw new RuntimeException("Error creating vertex shader.");
            }

            int fragmentShaderHandle = GLES20.glCreateShader(GLES20.GL_FRAGMENT_SHADER);
            if (fragmentShaderHandle != 0) {
                GLES20.glShaderSource(fragmentShaderHandle, filter_fragment_shader);
                GLES20.glCompileShader(fragmentShaderHandle);
                final int[] compileStatus = new int[1];
                GLES20.glGetShaderiv(fragmentShaderHandle, GLES20.GL_COMPILE_STATUS, compileStatus,
                        0);

                if (compileStatus[0] == 0) {
                    GLES20.glDeleteShader(fragmentShaderHandle);
                    Log.d("OpenGLES",
                            "Compilation\n" + GLES20.glGetShaderInfoLog(fragmentShaderHandle));
                    fragmentShaderHandle = 0;
                }
            }

            if (fragmentShaderHandle == 0) {
                Log.e(TAG, "filter_fragment_shader=" + filter_fragment_shader);
                throw new RuntimeException("Error creating fragment shader.");
            }

            int programHandle = GLES20.glCreateProgram();
            if (programHandle != 0) {
                GLES20.glAttachShader(programHandle, vertexShaderHandle);
                GLES20.glAttachShader(programHandle, fragmentShaderHandle);
                GLES20.glBindAttribLocation(programHandle, 0, "position");
                GLES20.glBindAttribLocation(programHandle, 1, "inputTextureCoordinate");
                GLES20.glLinkProgram(programHandle);
                final int[] linkStatus = new int[1];
                GLES20.glGetProgramiv(programHandle, GLES20.GL_LINK_STATUS, linkStatus, 0);
                if (DEBUG) {
                    Log.e(TAG, "linkStatus[0]=" + linkStatus[0]);
                }

                if (linkStatus[0] == 0) {
                    GLES20.glDeleteProgram(programHandle);
                    programHandle = 0;
                }
            }
            if (programHandle == 0) {

                throw new RuntimeException("Error creating program.");
            }
            mGLAttribPosition = GLES20.glGetAttribLocation(programHandle, "position");
            mGLAttribTextureCoordinate = GLES20.glGetAttribLocation(programHandle,
                    "inputTextureCoordinate");
            mGLUniformTexture = GLES20.glGetUniformLocation(programHandle, "inputImageTexture");
            mGLUniformUvTexture = GLES20.glGetUniformLocation(programHandle, "uvTexture");
            muMVPMatrixHandle = GLES20.glGetUniformLocation(programHandle, "uMVPMatrix");

            GLES20.glUseProgram(programHandle);
            mGLProgId = programHandle;
            mIsInitialized = true;
        } else {
            super.onInit();
        }

    }

    @Override
    public void showScreenSize(int height, int maxHeight, Rotation mRotation, int cameraId) {

        float ratioTemp = (float) mHeight / (mWidth * 1f);
        float ratio = mSurfaceRatio / ratioTemp;
        //
        //        if (mRotation == Rotation.ROTATION_270 || mRotation == Rotation.ROTATION_90) {
        //            ratio = 1f / ratio;
        //        }
        //
        //
        //        Log.e(TAG, "showScreenSize  mRotation=" + mRotation + "  ratio=" + ratio
        //                + "  mSurfaceRatio=" + mSurfaceRatio + " ratioTemp=" + ratioTemp);

        if (ratio > 1) {
            ratio = 1;
        }

        Matrix.frustumM(mProjMatrix, 0, -ratio, ratio, -1, 1, 1, 2);

        Matrix.setLookAtM(mVMatrix, 0,//
                0, 0, cameraId == Camera.CameraInfo.CAMERA_FACING_FRONT ? -1 : 1, //eye
                0f, 0f, 0f,//center
                0f, 1f, 0f//up
        );

        float y = ((maxHeight - height) / 1f / maxHeight);
        Matrix.translateM(mVMatrix, 0,//
                0, y, 0f//x,y,z
        );

        Matrix.rotateM(mVMatrix, 0, mRotation.asInt(), 0, 0, -1);
        //
        //        if (cameraId == Camera.CameraInfo.CAMERA_FACING_FRONT) {
        //            Matrix.rotateM(mVMatrix, 0, 180, 0, 0, -1);
        //        }

        Matrix.multiplyMM(mResultMatrix, 0, mProjMatrix, 0, mVMatrix, 0);
    }

    private void print(float[] vMatrix) {
        Log.e(TAG, "print \n\n\n");
        for (int i = 0; i < 4; i++) {
            Log.e(TAG, String.format("%s \t%s \t%s \t%s", vMatrix[i * 4], vMatrix[i * 4 + 1],
                    vMatrix[i * 4 + 2], vMatrix[i * 4 + 3]));
        }
        Log.e(TAG, "  \n\n");
    }

    @Override
    public void setPreviewFrame(byte[] data, float offsetX, float offsetY, int width, int height) {

        synchronized (lock) {
            if (mByteBuffer == null) {
                mByteBuffer = ByteBuffer.allocateDirect(mWidth * mHeight * 3 >> 1);
            }
            mWidth = width;
            mHeight = height;

            if (mByteBuffer != null && data != null) {
                hasPreview = true;
                mByteBuffer.clear();
                if (data.length <= mByteBuffer.limit() && data.length <= mByteBuffer.remaining()) {
                    mByteBuffer.put(data).position(0);
                }
            }
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mByteBuffer != null) {
            mByteBuffer.clear();
        }
        if (mVerticesBuffer != null) {
            mVerticesBuffer.clear();
        }
        if (mTextureCoordinateBuffer != null) {
            mTextureCoordinateBuffer.clear();
        }
    }
}
